@@ -256,6 +256,11 @@ struct aws_credentials_provider_ecs_options {
 
     /* For mocking the http layer in tests, leave NULL otherwise */
     struct aws_auth_http_system_vtable *function_table;
+
+    /*
+     * Port to query credentials from.  If zero, 80/443 will be used based on whether or not tls is enabled.
+     */
+    uint16_t port;
 };
 
 /**
@@ -458,6 +463,9 @@ AWS_EXTERN_C_BEGIN
 
 /*
  * Credentials APIs
+ *
+ * expiration_timepoint_seconds is the timepoint, in seconds since epoch, that the credentials will no longer
+ * be valid.  For credentials that do not expire, use UINT64_MAX.
  */
 
 /**
@@ -546,13 +554,22 @@ struct aws_byte_cursor aws_credentials_get_session_token(const struct aws_creden
 /**
  * Get the expiration timepoint (in seconds since epoch) associated with a set of credentials
  *
- * @param credentials redentials to get the expiration timepoint for
+ * @param credentials credentials to get the expiration timepoint for
  * @return the time, in seconds since epoch, the credentials will expire; UINT64_MAX for credentials
  * without a specific expiration time
  */
 AWS_AUTH_API
 uint64_t aws_credentials_get_expiration_timepoint_seconds(const struct aws_credentials *credentials);
 
+/**
+ * Get the elliptic curve key associated with this set of credentials
+ * @param credentials credentials to get the the elliptic curve key for
+ * @return the elliptic curve key associated with the credentials, or NULL if no key is associated with
+ * these credentials
+ */
+AWS_AUTH_API
+struct aws_ecc_key_pair *aws_credentials_get_ecc_key_pair(const struct aws_credentials *credentials);
+
 /*
  * Credentials provider APIs
  */
@@ -773,8 +790,9 @@ struct aws_credentials_provider *aws_credentials_provider_new_delegate(
  *
  * (1) Environment
  * (2) Profile
- * (3) (conditional, off by default) ECS
- * (4) (conditional, on by default) EC2 Instance Metadata
+ * (3) STS web identity
+ * (4) (conditional, off by default) ECS
+ * (5) (conditional, on by default) EC2 Instance Metadata
  *
  * Support for environmental control of the default provider chain is not yet
  * implemented.
@@ -789,6 +807,31 @@ struct aws_credentials_provider *aws_credentials_provider_new_chain_default(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_chain_default_options *options);
 
+AWS_AUTH_API
+struct aws_credentials *aws_credentials_new_ecc(
+    struct aws_allocator *allocator,
+    struct aws_byte_cursor access_key_id,
+    struct aws_ecc_key_pair *ecc_key,
+    struct aws_byte_cursor session_token,
+    uint64_t expiration_timepoint_in_seconds);
+
+/*
+ * Takes a pair of AWS credentials and performs the sigv4a key expansion algorithm to generate a unique
+ * ecc P256 key pair based on the credentials.  The ecc key is written to the buffer in DER format.
+ *
+ * Sigv4a signing takes the raw DER-encoded ecc key as an optional parameter in signing (if not present,
+ * key expansion will be done for the caller before signing).
+ */
+AWS_AUTH_API
+struct aws_credentials *aws_credentials_new_ecc_from_aws_credentials(
+    struct aws_allocator *allocator,
+    const struct aws_credentials *credentials);
+
+AWS_AUTH_API
+struct aws_ecc_key_pair *aws_ecc_key_pair_new_ecdsa_p256_key_from_aws_credentials(
+    struct aws_allocator *allocator,
+    const struct aws_credentials *credentials);
+
 AWS_EXTERN_C_END
 
 #endif /* AWS_AUTH_CREDENTIALS_H */
