@@ -14,8 +14,11 @@
 #    define __BSD_VISIBLE 1
 #endif
 
-#if defined(__linux__) || defined(__unix__)
+#if defined(__linux__)
 #    include <sys/sysinfo.h>
+#endif
+
+#if defined(__linux__) || defined(__unix__)
 #    include <sys/types.h>
 #endif
 
@@ -450,40 +453,3 @@ enum aws_platform_os aws_get_platform_build_os(void) {
     return AWS_PLATFORM_OS_UNIX;
 }
 #endif /* AWS_OS_APPLE */
-
-#if defined(__linux__) || defined(__unix__)
-static const uint64_t S_BYTES_TO_KILO_BYTES = 1024;
-#endif
-
-int aws_get_memory_usage(int64_t *output) {
-// Get the Memory usage from Linux
-#if defined(__linux__) || defined(__unix__)
-    struct sysinfo memory_info;
-    sysinfo(&memory_info);
-    uint64_t physical_memory_used = memory_info.totalram - memory_info.freeram;
-    physical_memory_used *= memory_info.mem_unit;
-    // Return data in Kilobytes
-    physical_memory_used = physical_memory_used / S_BYTES_TO_KILO_BYTES;
-    *output = physical_memory_used;
-    return AWS_OP_SUCCESS;
-#endif
-
-    // OS not supported? Just return an error and set the output to 0
-    *output = 0;
-    aws_raise_error(AWS_ERROR_PLATFORM_NOT_SUPPORTED);
-    return AWS_OP_ERR;
-}
-
-int aws_get_process_count(int64_t *output) {
-// Get the process count from Linux
-#if defined(__linux__) || defined(__unix__)
-    struct sysinfo system_info;
-    sysinfo(&system_info);
-    *output = (int64_t)system_info.procs;
-    return AWS_OP_SUCCESS;
-#endif
-    // OS not supported? Just return an error and set the output to 0
-    *output = 0;
-    aws_raise_error(AWS_ERROR_PLATFORM_NOT_SUPPORTED);
-    return AWS_OP_ERR;
-}
