@@ -18,6 +18,7 @@ struct proxy_env_var_settings;
 struct aws_http2_setting;
 struct aws_http_make_request_options;
 struct aws_http_stream;
+struct aws_http_manager_metrics;
 
 /**
  * Always invoked asynchronously when the stream was created, successfully or not.
@@ -47,12 +48,23 @@ struct aws_http2_stream_manager_options {
      */
     struct aws_client_bootstrap *bootstrap;
     const struct aws_socket_options *socket_options;
+
     /**
-     * If TLS options is set, you also need to handle ALPN, otherwise, may not able to get HTTP/2 connection and fail
-     * the stream manager.
-     * If TLS options not set, prior knowledge will be used.
+     * Options to create secure (HTTPS) connections.
+     * For secure connections, the ALPN string must be "h2".
+     *
+     * To create cleartext (HTTP) connections, leave this NULL
+     * and set `http2_prior_knowledge` (RFC-7540 3.4).
      */
     const struct aws_tls_connection_options *tls_connection_options;
+
+    /**
+     * Specify whether you have prior knowledge that cleartext (HTTP) connections are HTTP/2 (RFC-7540 3.4).
+     * It is illegal to set this true when secure connections are being used.
+     * Note that upgrading from HTTP/1.1 to HTTP/2 is not supported (RFC-7540 3.2).
+     */
+    bool http2_prior_knowledge;
+
     struct aws_byte_cursor host;
     uint16_t port;
 
@@ -63,8 +75,8 @@ struct aws_http2_stream_manager_options {
      * - By default, client will will maintain its flow-control windows such that no back-pressure is applied and data
      * arrives as fast as possible.
      * - For connection level window control, `conn_manual_window_management` will enable manual control. The
-     * inital window size is not controlable.
-     * - For stream level window control, `enable_read_back_pressure` will enable manual control. The initail window
+     * inital window size is not controllable.
+     * - For stream level window control, `enable_read_back_pressure` will enable manual control. The initial window
      * size needs to be set through `initial_settings_array`.
      */
     struct aws_http2_setting *initial_settings_array;
@@ -93,6 +105,26 @@ struct aws_http2_stream_manager_options {
     void *shutdown_complete_user_data;
     aws_http2_stream_manager_shutdown_complete_fn *shutdown_complete_callback;
 
+    /**
+     * Optional.
+     * When set, connection will be closed if 5xx response received from server.
+     */
+    bool close_connection_on_server_error;
+    /**
+     * Optional.
+     * The period for all the connections held by stream manager to send a PING in milliseconds.
+     * If you specify 0, manager will NOT send any PING.
+     * Note: if set, it must be large than the time of ping timeout setting.
+     */
+    size_t connection_ping_period_ms;
+    /**
+     * Optional.
+     * Network connection will be closed if a ping response is not received
+     * within this amount of time (milliseconds).
+     * If you specify 0, a default value will be used.
+     */
+    size_t connection_ping_timeout_ms;
+
     /* TODO: More flexible policy about the connections, but will always has these three values below. */
     /**
      * Optional.
@@ -168,5 +200,16 @@ void aws_http2_stream_manager_acquire_stream(
     struct aws_http2_stream_manager *http2_stream_manager,
     const struct aws_http2_stream_manager_acquire_stream_options *acquire_stream_option);
 
+/**
+ * Fetch the current metrics from stream manager.
+ *
+ * @param http2_stream_manager
+ * @param out_metrics The metrics to be fetched
+ */
+AWS_HTTP_API
+void aws_http2_stream_manager_fetch_metrics(
+    const struct aws_http2_stream_manager *http2_stream_manager,
+    struct aws_http_manager_metrics *out_metrics);
+
 AWS_EXTERN_C_END
 #endif /* AWS_HTTP2_STREAM_MANAGER_H */
