@@ -751,6 +751,17 @@ int aws_http1_stream_write_chunk(struct aws_http_stream *http1_stream, const str
     return http1_stream->vtable->http1_write_chunk(http1_stream, options);
 }
 
+int aws_http2_stream_write_data(
+    struct aws_http_stream *http2_stream,
+    const struct aws_http2_stream_write_data_options *options) {
+    AWS_PRECONDITION(http2_stream);
+    AWS_PRECONDITION(http2_stream->vtable);
+    AWS_PRECONDITION(http2_stream->vtable->http2_write_data);
+    AWS_PRECONDITION(options);
+
+    return http2_stream->vtable->http2_write_data(http2_stream, options);
+}
+
 int aws_http1_stream_add_chunked_trailer(
     struct aws_http_stream *http1_stream,
     const struct aws_http_headers *trailing_headers) {
@@ -773,7 +784,7 @@ struct aws_input_stream *aws_http_message_get_body_stream(const struct aws_http_
     return message->body_stream;
 }
 
-struct aws_http_headers *aws_http_message_get_headers(struct aws_http_message *message) {
+struct aws_http_headers *aws_http_message_get_headers(const struct aws_http_message *message) {
     AWS_PRECONDITION(message);
     return message->headers;
 }
@@ -841,8 +852,8 @@ struct aws_http_stream *aws_http_connection_make_request(
 }
 
 struct aws_http_message *aws_http2_message_new_from_http1(
-    struct aws_http_message *http1_msg,
-    struct aws_allocator *alloc) {
+    struct aws_allocator *alloc,
+    const struct aws_http_message *http1_msg) {
 
     struct aws_http_headers *old_headers = aws_http_message_get_headers(http1_msg);
     struct aws_http_header header_iter;
@@ -850,11 +861,12 @@ struct aws_http_message *aws_http2_message_new_from_http1(
     AWS_ZERO_STRUCT(lower_name_buf);
     struct aws_http_message *message = aws_http_message_is_request(http1_msg) ? aws_http2_message_new_request(alloc)
                                                                               : aws_http2_message_new_response(alloc);
-    struct aws_http_headers *copied_headers = message->headers;
     if (!message) {
         return NULL;
     }
-    AWS_LOGF_TRACE(AWS_LS_HTTP_GENERAL, "Creating HTTP/2 message from HTTP/1 message id: %p", (void *)http1_msg)
+    struct aws_http_headers *copied_headers = message->headers;
+    AWS_LOGF_TRACE(AWS_LS_HTTP_GENERAL, "Creating HTTP/2 message from HTTP/1 message id: %p", (void *)http1_msg);
+
     /* Set pseudo headers from HTTP/1.1 message */
     if (aws_http_message_is_request(http1_msg)) {
         struct aws_byte_cursor method;
@@ -867,7 +879,7 @@ struct aws_http_message *aws_http2_message_new_from_http1(
             aws_raise_error(AWS_ERROR_HTTP_INVALID_METHOD);
             goto error;
         }
-        /* Use add intead of set method to avoid push front to the array list */
+        /* Use add instead of set method to avoid push front to the array list */
         if (aws_http_headers_add(copied_headers, aws_http_header_method, method)) {
             goto error;
         }
@@ -893,7 +905,28 @@ struct aws_http_message *aws_http2_message_new_from_http1(
             aws_http_header_scheme.ptr,
             (int)scheme_cursor.len,
             scheme_cursor.ptr);
-        /* :authority SHOULD NOT be created when translating HTTP/1 request.(RFC 7540 8.1.2.3) */
+
+        /**
+         * An intermediary that forwards a request over HTTP/2 MUST construct an ":authority" pseudo-header field using
+         * the authority information from the control data of the original request. (RFC=9113 8.3.1)
+         */
+        struct aws_byte_cursor host_value;
+        AWS_ZERO_STRUCT(host_value);
+        if (aws_http_headers_get(http1_msg->headers, aws_byte_cursor_from_c_str("host"), &host_value) ==
+            AWS_OP_SUCCESS) {
+            if (aws_http_headers_add(copied_headers, aws_http_header_authority, host_value)) {
+                goto error;
+            }
+            AWS_LOGF_TRACE(
+                AWS_LS_HTTP_GENERAL,
+                "Added header to new HTTP/2 header - \"%.*s\": \"%.*s\" ",
+                (int)aws_http_header_authority.len,
+                aws_http_header_authority.ptr,
+                (int)host_value.len,
+                host_value.ptr);
+        }
+        /* TODO: If the host headers is missing, the target URI could be the other source of the authority information
+         */
 
         struct aws_byte_cursor path_cursor;
         if (aws_http_message_get_request_path(http1_msg, &path_cursor)) {
@@ -941,6 +974,8 @@ struct aws_http_message *aws_http2_message_new_from_http1(
         goto error;
     }
     for (size_t iter = 0; iter < aws_http_headers_count(old_headers); iter++) {
+        aws_byte_buf_reset(&lower_name_buf, false);
+        bool copy_header = true;
         /* name should be converted to lower case */
         if (aws_http_headers_get_index(old_headers, iter, &header_iter)) {
             goto error;
@@ -948,23 +983,42 @@ struct aws_http_message *aws_http2_message_new_from_http1(
         /* append lower case name to the buffer */
         aws_byte_buf_append_with_lookup(&lower_name_buf, &header_iter.name, aws_lookup_table_to_lower_get());
         struct aws_byte_cursor lower_name_cursor = aws_byte_cursor_from_buf(&lower_name_buf);
+        enum aws_http_header_name name_enum = aws_http_lowercase_str_to_header_name(lower_name_cursor);
+        switch (name_enum) {
+            case AWS_HTTP_HEADER_TRANSFER_ENCODING:
+            case AWS_HTTP_HEADER_UPGRADE:
+            case AWS_HTTP_HEADER_KEEP_ALIVE:
+            case AWS_HTTP_HEADER_PROXY_CONNECTION:
+                /**
+                 * An intermediary transforming an HTTP/1.x message to HTTP/2 MUST remove connection-specific header
+                 * fields as discussed in Section 7.6.1 of [HTTP]. (RFC=9113 8.2.2)
+                 */
+                AWS_LOGF_TRACE(
+                    AWS_LS_HTTP_GENERAL,
+                    "Skip connection-specific headers - \"%.*s\" ",
+                    (int)lower_name_cursor.len,
+                    lower_name_cursor.ptr);
+                copy_header = false;
+                break;
 
-        /* TODO: handle connection-specific header field (RFC7540 8.1.2.2) */
-        if (aws_http_headers_add(copied_headers, lower_name_cursor, header_iter.value)) {
-            goto error;
+            default:
+                break;
+        }
+        if (copy_header) {
+            if (aws_http_headers_add(copied_headers, lower_name_cursor, header_iter.value)) {
+                goto error;
+            }
+            AWS_LOGF_TRACE(
+                AWS_LS_HTTP_GENERAL,
+                "Added header to new HTTP/2 header - \"%.*s\": \"%.*s\" ",
+                (int)lower_name_cursor.len,
+                lower_name_cursor.ptr,
+                (int)header_iter.value.len,
+                header_iter.value.ptr);
         }
-        AWS_LOGF_TRACE(
-            AWS_LS_HTTP_GENERAL,
-            "Added header to new HTTP/2 header - \"%.*s\": \"%.*s\" ",
-            (int)lower_name_cursor.len,
-            lower_name_cursor.ptr,
-            (int)header_iter.value.len,
-            header_iter.value.ptr);
-        aws_byte_buf_reset(&lower_name_buf, false);
     }
     aws_byte_buf_clean_up(&lower_name_buf);
     aws_http_message_set_body_stream(message, aws_http_message_get_body_stream(http1_msg));
-    /* TODO: Refcount the input stream of old message */
 
     return message;
 error:
