@@ -23,6 +23,8 @@ struct aws_s3_client;
 struct aws_s3_request;
 struct aws_s3_meta_request;
 struct aws_s3_meta_request_result;
+struct aws_uri;
+struct aws_string;
 
 /**
  * A Meta Request represents a group of generated requests that are being done on behalf of the
@@ -51,6 +53,14 @@ enum aws_s3_meta_request_type {
      */
     AWS_S3_META_REQUEST_TYPE_PUT_OBJECT,
 
+    /**
+     * The CopyObject meta request performs a multi-part copy
+     * using multiple S3 UploadPartCopy requests in parallel, or bypasses
+     * a CopyObject request to S3 if the object size is not large enough for
+     * a multipart upload.
+     */
+    AWS_S3_META_REQUEST_TYPE_COPY_OBJECT,
+
     AWS_S3_META_REQUEST_TYPE_MAX,
 };
 
@@ -74,6 +84,7 @@ typedef int(aws_s3_meta_request_headers_callback_fn)(
  * Return AWS_OP_ERR to indicate failure and cancel the request.
  */
 typedef int(aws_s3_meta_request_receive_body_callback_fn)(
+
     /* The meta request that the callback is being issued for. */
     struct aws_s3_meta_request *meta_request,
 
@@ -95,6 +106,26 @@ typedef void(aws_s3_meta_request_finish_fn)(
     const struct aws_s3_meta_request_result *meta_request_result,
     void *user_data);
 
+/**
+ * Information sent in the meta_request progress callback.
+ */
+struct aws_s3_meta_request_progress {
+
+    /* Bytes transferred since the previous progress update */
+    uint64_t bytes_transferred;
+
+    /* Length of the entire meta request operation */
+    uint64_t content_length;
+};
+
+/**
+ * Invoked to report progress of multi-part upload and copy object requests.
+ */
+typedef void(aws_s3_meta_request_progress_fn)(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_s3_meta_request_progress *progress,
+    void *user_data);
+
 typedef void(aws_s3_meta_request_shutdown_fn)(void *user_data);
 
 typedef void(aws_s3_client_shutdown_complete_callback_fn)(void *user_data);
@@ -109,6 +140,16 @@ enum aws_s3_meta_request_compute_content_md5 {
     AWS_MR_CONTENT_MD5_ENABLED,
 };
 
+enum aws_s3_checksum_algorithm {
+    AWS_SCA_NONE = 0,
+    AWS_SCA_INIT,
+    AWS_SCA_CRC32C = AWS_SCA_INIT,
+    AWS_SCA_CRC32,
+    AWS_SCA_SHA1,
+    AWS_SCA_SHA256,
+    AWS_SCA_COUNT,
+};
+
 /* Options for a new client. */
 struct aws_s3_client_config {
 
@@ -174,6 +215,32 @@ struct aws_s3_meta_request_options {
     /* Initial HTTP message that defines what operation we are doing. */
     struct aws_http_message *message;
 
+    /**
+     * checksum algorithm will cause single part put requests to append a trailing checksum of the body corresponding to
+     * the specified algorithm. Multipart uploads will have a checksum uploaded corresponding to each individual part,
+     * and the body of the complete multipart upload request will contain a list of the checksums from erach part.
+     * Get requests will be unaffected by this setting. Copy object requests will have a header specifying to s3 to
+     * calculate a new checksum with the corresponding algorithm.
+     * Note: If set, it will disable compute_content_md5
+     */
+    /* TODO add functionality for copy object */
+    enum aws_s3_checksum_algorithm checksum_algorithm;
+
+    /**
+     * Enable checksumode header will be attached to get requests, this will tell s3 to send back checksums headers if
+     * they exist.
+     *
+     * - If the object to get was uploaded as a single part upload with checksums, s3 will provide a checksum of the
+     * entire object. The checksum of the entire body received will be calculated and compared to the checksum provided
+     * by s3.
+     * - If a checksum of a part object is received, the client will
+     * calculate the corresponding checksum on the response bodies. The metarequest will finish with a did
+     * validate field and set the error code to AWS_ERROR_S3_RESPONSE_CHECKSUM_MISMATCH if the calculated
+     * checksum, and checksum found in the response header do not match.
+     */
+    /* TODO add a validation list, empty validation list is equivalent to false */
+    bool validate_get_response_checksum;
+
     /* User data for all callbacks. */
     void *user_data;
 
@@ -198,6 +265,31 @@ struct aws_s3_meta_request_options {
 
     /* Callback for when the meta request has completely cleaned up. */
     aws_s3_meta_request_shutdown_fn *shutdown_callback;
+
+    /**
+     * Invoked to report progress of the meta request execution.
+     * Currently, the progress callback is invoked only for the CopyObject meta request type.
+     * TODO: support this callback for all the types of meta requests
+     * See `aws_s3_meta_request_progress_fn`
+     */
+    aws_s3_meta_request_progress_fn *progress_callback;
+
+    /**
+     * Optional.
+     * The endpoint for the meta request to connect to. If not set, then tls settings from client will
+     * determine the port, and host header from `message` will determine the host for the connection.
+     * Note: must match the host header from `message`
+     */
+    struct aws_uri *endpoint;
+
+    /**
+     * Optional.
+     * For meta requests that support pause/resume (e.g. PutObject), the resume token returned by
+     * aws_s3_meta_request_pause() can be provided here.
+     * Note: If PutObject request specifies a checksum algorithm, client will calculate checksums while skipping parts
+     * from the buffer and compare them them to previously uploaded part checksums.
+     */
+    const struct aws_byte_cursor *resume_token;
 };
 
 /* Result details of a meta request.
@@ -221,6 +313,15 @@ struct aws_s3_meta_request_result {
     /* Response status of the failed request or of the entire meta request. */
     int response_status;
 
+    /* Only set for GET request.
+     * Was the server side checksum compared against a calculated checksum of the response body. This may be false
+     * even if validate_get_response_checksum was set because the object was uploaded without a checksum, or was
+     * uploaded as a multipart object. */
+    bool did_validate;
+
+    /* algorithm used to validate checksum */
+    enum aws_s3_checksum_algorithm validation_algorithm;
+
     /* Final error code of the meta request. */
     int error_code;
 };
@@ -246,6 +347,21 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
 AWS_S3_API
 void aws_s3_meta_request_cancel(struct aws_s3_meta_request *meta_request);
 
+/**
+ * In order to pause an ongoing upload, call aws_s3_meta_request_pause(). It will return a resume token that can be
+ * persisted and used to resume the upload. To resume an upload that was paused, supply the resume token in the meta
+ * request options structure member aws_s3_meta_request_options.persistable_state.
+ * The upload can be resumed either from the same client or a different one.
+ * Resume token is opaque with format varying based on operation.
+ * Clients should not parse the token. For format details refer to pause method comments for a given operation.
+ * Resume token will be set to null in case of failures.
+ * @param meta_request pointer to the aws_s3_meta_request of the upload to be paused
+ * @param resume_token outputs the json string with the state that can be used to resume the operation.
+ * @return
+ */
+AWS_S3_API
+int aws_s3_meta_request_pause(struct aws_s3_meta_request *meta_request, struct aws_string **out_resume_token);
+
 AWS_S3_API
 void aws_s3_meta_request_acquire(struct aws_s3_meta_request *meta_request);
 
