@@ -134,7 +134,7 @@ void aws_s3_set_dns_ttl(size_t ttl) {
  * When meta request is NULL, this will return the overall allowed number of connections.
  *
  * If meta_request is not NULL, this will give the max number of connections allowed for that meta request type on
- * thatendpoint.
+ * that endpoint.
  */
 uint32_t aws_s3_client_get_max_active_connections(
     struct aws_s3_client *client,
@@ -285,35 +285,6 @@ struct aws_s3_client *aws_s3_client_new(
 
     client->process_work_event_loop = aws_event_loop_group_get_next_loop(event_loop_group);
 
-    /* Set up body streaming ELG */
-    {
-        uint16_t num_event_loops =
-            (uint16_t)aws_array_list_length(&client->client_bootstrap->event_loop_group->event_loops);
-        uint16_t num_streaming_threads = num_event_loops;
-
-        if (num_streaming_threads < 1) {
-            num_streaming_threads = 1;
-        }
-
-        struct aws_shutdown_callback_options body_streaming_elg_shutdown_options = {
-            .shutdown_callback_fn = s_s3_client_body_streaming_elg_shutdown,
-            .shutdown_callback_user_data = client,
-        };
-
-        if (aws_get_cpu_group_count() > 1) {
-            client->body_streaming_elg = aws_event_loop_group_new_default_pinned_to_cpu_group(
-                client->allocator, num_streaming_threads, 1, &body_streaming_elg_shutdown_options);
-        } else {
-            client->body_streaming_elg = aws_event_loop_group_new_default(
-                client->allocator, num_streaming_threads, &body_streaming_elg_shutdown_options);
-        }
-        if (!client->body_streaming_elg) {
-            /* Fail to create elg, we should fail the call */
-            goto elg_create_fail;
-        }
-        client->synced_data.body_streaming_elg_allocated = true;
-    }
-
     /* Make a copy of the region string. */
     client->region = aws_string_new_from_array(allocator, client_config->region.ptr, client_config->region.len);
 
@@ -362,6 +333,36 @@ struct aws_s3_client *aws_s3_client_new(
         }
     }
 
+    /* Set up body streaming ELG */
+    {
+        uint16_t num_event_loops =
+            (uint16_t)aws_array_list_length(&client->client_bootstrap->event_loop_group->event_loops);
+        uint16_t num_streaming_threads = num_event_loops;
+
+        if (num_streaming_threads < 1) {
+            num_streaming_threads = 1;
+        }
+
+        struct aws_shutdown_callback_options body_streaming_elg_shutdown_options = {
+            .shutdown_callback_fn = s_s3_client_body_streaming_elg_shutdown,
+            .shutdown_callback_user_data = client,
+        };
+
+        if (aws_get_cpu_group_count() > 1) {
+            client->body_streaming_elg = aws_event_loop_group_new_default_pinned_to_cpu_group(
+                client->allocator, num_streaming_threads, 1, &body_streaming_elg_shutdown_options);
+        } else {
+            client->body_streaming_elg = aws_event_loop_group_new_default(
+                client->allocator, num_streaming_threads, &body_streaming_elg_shutdown_options);
+        }
+        if (!client->body_streaming_elg) {
+            /* Fail to create elg, we should fail the call */
+            goto on_error;
+        }
+        client->synced_data.body_streaming_elg_allocated = true;
+    }
+    /* Setup cannot fail after this point. */
+
     if (client_config->throughput_target_gbps != 0.0) {
         *((double *)&client->throughput_target_gbps) = client_config->throughput_target_gbps;
     } else {
@@ -415,14 +416,11 @@ struct aws_s3_client *aws_s3_client_new(
     return client;
 
 on_error:
-    aws_event_loop_group_release(client->body_streaming_elg);
-    client->body_streaming_elg = NULL;
     if (client->tls_connection_options) {
         aws_tls_connection_options_clean_up(client->tls_connection_options);
         aws_mem_release(client->allocator, client->tls_connection_options);
         client->tls_connection_options = NULL;
     }
-elg_create_fail:
     aws_event_loop_group_release(client->client_bootstrap->event_loop_group);
     aws_client_bootstrap_release(client->client_bootstrap);
     aws_mutex_clean_up(&client->synced_data.lock);
@@ -760,13 +758,14 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
     bool content_length_header_found = false;
 
     if (!aws_http_headers_get(initial_message_headers, g_content_length_header_name, &content_length_cursor)) {
-        struct aws_string *content_length_str = aws_string_new_from_cursor(client->allocator, &content_length_cursor);
-        char *content_length_str_end = NULL;
-
-        content_length = strtoull((const char *)content_length_str->bytes, &content_length_str_end, 10);
-        aws_string_destroy(content_length_str);
-
-        content_length_str = NULL;
+        if (aws_byte_cursor_utf8_parse_u64(content_length_cursor, &content_length)) {
+            AWS_LOGF_ERROR(
+                AWS_LS_S3_META_REQUEST,
+                "Could not parse Content-Length header. header value is:" PRInSTR "",
+                AWS_BYTE_CURSOR_PRI(content_length_cursor));
+            aws_raise_error(AWS_ERROR_S3_INVALID_CONTENT_LENGTH_HEADER);
+            return NULL;
+        }
         content_length_header_found = true;
     }
 
@@ -807,81 +806,86 @@ static struct aws_s3_meta_request *s_s3_client_meta_request_factory_default(
             return NULL;
         }
 
-        size_t client_part_size = client->part_size;
-        size_t client_max_part_size = client->max_part_size;
+        if (options->resume_token == NULL) {
 
-        if (client_part_size < g_s3_min_upload_part_size) {
-            AWS_LOGF_WARN(
-                AWS_LS_S3_META_REQUEST,
-                "Client config part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
-                ". Using to the minimum part-size for upload.",
-                (uint64_t)client_part_size,
-                (uint64_t)g_s3_min_upload_part_size);
+            size_t client_part_size = client->part_size;
+            size_t client_max_part_size = client->max_part_size;
 
-            client_part_size = g_s3_min_upload_part_size;
+            if (client_part_size < g_s3_min_upload_part_size) {
+                AWS_LOGF_WARN(
+                    AWS_LS_S3_META_REQUEST,
+                    "Client config part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
+                    ". Using to the minimum part-size for upload.",
+                    (uint64_t)client_part_size,
+                    (uint64_t)g_s3_min_upload_part_size);
+
+                client_part_size = g_s3_min_upload_part_size;
+            }
+
+            if (client_max_part_size < g_s3_min_upload_part_size) {
+                AWS_LOGF_WARN(
+                    AWS_LS_S3_META_REQUEST,
+                    "Client config max part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
+                    ". Clamping to the minimum part-size for upload.",
+                    (uint64_t)client_max_part_size,
+                    (uint64_t)g_s3_min_upload_part_size);
+
+                client_max_part_size = g_s3_min_upload_part_size;
+            }
+            if (content_length <= client_part_size) {
+                return aws_s3_meta_request_default_new(
+                    client->allocator,
+                    client,
+                    content_length,
+                    client->compute_content_md5 == AWS_MR_CONTENT_MD5_ENABLED &&
+                        !aws_http_headers_has(initial_message_headers, g_content_md5_header_name),
+                    options,
+                    options->checksum_algorithm,
+                    false);
+            }
+
+            uint64_t part_size_uint64 = content_length / (uint64_t)g_s3_max_num_upload_parts;
+
+            if (part_size_uint64 > SIZE_MAX) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "Could not create auto-ranged-put meta request; required part size of %" PRIu64
+                    " bytes is too large for platform.",
+                    part_size_uint64);
+
+                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                return NULL;
+            }
+
+            size_t part_size = (size_t)part_size_uint64;
+
+            if (part_size > client_max_part_size) {
+                AWS_LOGF_ERROR(
+                    AWS_LS_S3_META_REQUEST,
+                    "Could not create auto-ranged-put meta request; required part size for put request is %" PRIu64
+                    ", but current maximum part size is %" PRIu64,
+                    (uint64_t)part_size,
+                    (uint64_t)client_max_part_size);
+                aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
+                return NULL;
+            }
+
+            if (part_size < client_part_size) {
+                part_size = client_part_size;
+            }
+
+            uint32_t num_parts = (uint32_t)(content_length / part_size);
+
+            if ((content_length % part_size) > 0) {
+                ++num_parts;
+            }
+
+            return aws_s3_meta_request_auto_ranged_put_new(
+                client->allocator, client, part_size, content_length, num_parts, options);
+        } else {
+            /* dont pass part size and total num parts. constructor will pick it up from token */
+            return aws_s3_meta_request_auto_ranged_put_new(client->allocator, client, 0, content_length, 0, options);
         }
-
-        if (client_max_part_size < g_s3_min_upload_part_size) {
-            AWS_LOGF_WARN(
-                AWS_LS_S3_META_REQUEST,
-                "Client config max part size of %" PRIu64 " is less than the minimum upload part size of %" PRIu64
-                ". Clamping to the minimum part-size for upload.",
-                (uint64_t)client_max_part_size,
-                (uint64_t)g_s3_min_upload_part_size);
-
-            client_max_part_size = g_s3_min_upload_part_size;
-        }
-
-        if (content_length <= client_part_size) {
-            return aws_s3_meta_request_default_new(
-                client->allocator,
-                client,
-                content_length,
-                client->compute_content_md5 == AWS_MR_CONTENT_MD5_ENABLED &&
-                    !aws_http_headers_has(initial_message_headers, g_content_md5_header_name),
-                options,
-                options->checksum_algorithm,
-                false);
-        }
-
-        uint64_t part_size_uint64 = content_length / (uint64_t)g_s3_max_num_upload_parts;
-
-        if (part_size_uint64 > SIZE_MAX) {
-            AWS_LOGF_ERROR(
-                AWS_LS_S3_META_REQUEST,
-                "Could not create auto-ranged-put meta request; required part size of %" PRIu64
-                " bytes is too large for platform.",
-                part_size_uint64);
-
-            aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-            return NULL;
-        }
-
-        size_t part_size = (size_t)part_size_uint64;
-
-        if (part_size > client_max_part_size) {
-            AWS_LOGF_ERROR(
-                AWS_LS_S3_META_REQUEST,
-                "Could not create auto-ranged-put meta request; required part size for put request is %" PRIu64
-                ", but current maximum part size is %" PRIu64,
-                (uint64_t)part_size,
-                (uint64_t)client_max_part_size);
-            aws_raise_error(AWS_ERROR_INVALID_ARGUMENT);
-            return NULL;
-        }
-
-        if (part_size < client_part_size) {
-            part_size = client_part_size;
-        }
-
-        uint32_t num_parts = (uint32_t)(content_length / part_size);
-
-        if ((content_length % part_size) > 0) {
-            ++num_parts;
-        }
-
-        return aws_s3_meta_request_auto_ranged_put_new(
-            client->allocator, client, part_size, content_length, num_parts, options);
     } else if (options->type == AWS_S3_META_REQUEST_TYPE_COPY_OBJECT) {
         return aws_s3_meta_request_copy_object_new(client->allocator, client, options);
     } else if (options->type == AWS_S3_META_REQUEST_TYPE_DEFAULT) {
