@@ -275,6 +275,19 @@ void aws_s3_meta_request_cancel(struct aws_s3_meta_request *meta_request) {
     /* END CRITICAL SECTION */
 }
 
+int aws_s3_meta_request_pause(struct aws_s3_meta_request *meta_request, struct aws_string **out_resume_token) {
+    AWS_PRECONDITION(meta_request);
+    AWS_PRECONDITION(meta_request->vtable);
+
+    *out_resume_token = NULL;
+
+    if (!meta_request->vtable->pause) {
+        return aws_raise_error(AWS_ERROR_UNSUPPORTED_OPERATION);
+    }
+
+    return meta_request->vtable->pause(meta_request, out_resume_token);
+}
+
 void aws_s3_meta_request_set_fail_synced(
     struct aws_s3_meta_request *meta_request,
     struct aws_s3_request *failed_request,
@@ -1318,13 +1331,15 @@ static struct aws_s3_request *s_s3_meta_request_body_streaming_pop_next_synced(
     AWS_PRECONDITION(meta_request);
     ASSERT_SYNCED_DATA_LOCK_HELD(meta_request);
 
+    if (0 == aws_priority_queue_size(&meta_request->synced_data.pending_body_streaming_requests)) {
+        return NULL;
+    }
+
     struct aws_s3_request **top_request = NULL;
 
     aws_priority_queue_top(&meta_request->synced_data.pending_body_streaming_requests, (void **)&top_request);
 
-    if (top_request == NULL) {
-        return NULL;
-    }
+    AWS_ASSERT(top_request);
 
     AWS_FATAL_ASSERT(*top_request);
 
