@@ -90,6 +90,39 @@ const struct aws_byte_cursor g_s3_complete_multipart_upload_excluded_headers[] =
 const size_t g_s3_complete_multipart_upload_excluded_headers_count =
     AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_excluded_headers);
 
+const struct aws_byte_cursor g_s3_list_parts_excluded_headers[] = {
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-acl"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Cache-Control"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Disposition"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Encoding"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Language"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Length"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-MD5"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Content-Type"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Expires"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-grant-full-control"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-grant-read"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-grant-read-acp"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-grant-write-acp"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-storage-class"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-website-redirect-location"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-algorithm"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-key"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-customer-key-MD5"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-aws-kms-key-id"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-context"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-server-side-encryption-bucket-key-enabled"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-tagging"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-mode"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-retain-until-date"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-object-lock-legal-hold"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
+};
+
+const size_t g_s3_list_parts_excluded_headers_count = AWS_ARRAY_SIZE(g_s3_list_parts_excluded_headers);
+
 const struct aws_byte_cursor g_s3_abort_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-acl"),
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("Cache-Control"),
@@ -121,6 +154,8 @@ const struct aws_byte_cursor g_s3_abort_multipart_upload_excluded_headers[] = {
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source-range"),
 };
 
+static const struct aws_byte_cursor s_x_amz_meta_prefix = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-meta-");
+
 const size_t g_s3_abort_multipart_upload_excluded_headers_count =
     AWS_ARRAY_SIZE(g_s3_abort_multipart_upload_excluded_headers);
 
@@ -139,7 +174,8 @@ struct aws_http_message *aws_s3_ranged_get_object_message_new(
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(allocator, base_message, NULL, 0);
+    struct aws_http_message *message =
+        aws_s3_message_util_copy_http_message_no_body_all_headers(allocator, base_message);
 
     if (message == NULL) {
         return NULL;
@@ -168,12 +204,16 @@ struct aws_http_message *aws_s3_create_multipart_upload_message_new(
     enum aws_s3_checksum_algorithm algorithm) {
     AWS_PRECONDITION(allocator);
 
-    /* For multipart upload, sse related headers should only be shown in create-multipart request */
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
+    /* For multipart upload, some headers should ONLY be in the initial create-multipart request.
+     * Headers such as:
+     * - SSE related headers
+     * - user metadata (prefixed "x-amz-meta-") headers */
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
         allocator,
         base_message,
         g_s3_create_multipart_upload_excluded_headers,
-        AWS_ARRAY_SIZE(g_s3_create_multipart_upload_excluded_headers));
+        AWS_ARRAY_SIZE(g_s3_create_multipart_upload_excluded_headers),
+        false /*exclude_x_amz_meta*/);
 
     if (message == NULL) {
         goto error_clean_up;
@@ -233,8 +273,12 @@ struct aws_http_message *aws_s3_upload_part_message_new(
     AWS_PRECONDITION(base_message);
     AWS_PRECONDITION(part_number > 0);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
-        allocator, base_message, g_s3_upload_part_excluded_headers, AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers));
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
+        allocator,
+        base_message,
+        g_s3_upload_part_excluded_headers,
+        AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers),
+        true /*exclude_x_amz_meta*/);
 
     if (message == NULL) {
         goto error_clean_up;
@@ -284,8 +328,12 @@ struct aws_http_message *aws_s3_upload_part_copy_message_new(
     AWS_PRECONDITION(base_message);
     AWS_PRECONDITION(part_number > 0);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
-        allocator, base_message, g_s3_upload_part_excluded_headers, AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers));
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
+        allocator,
+        base_message,
+        g_s3_upload_part_excluded_headers,
+        AWS_ARRAY_SIZE(g_s3_upload_part_excluded_headers),
+        true /*exclude_x_amz_meta*/);
 
     if (message == NULL) {
         goto error_clean_up;
@@ -401,32 +449,19 @@ struct aws_http_message *aws_s3_get_source_object_size_message_new(
     /* find the x-amz-copy-source header */
     struct aws_http_headers *headers = aws_http_message_get_headers(base_message);
 
-    struct aws_byte_cursor source_header_value;
-    AWS_ZERO_STRUCT(source_header_value);
+    struct aws_byte_cursor source_bucket;
+    AWS_ZERO_STRUCT(source_bucket);
 
     const struct aws_byte_cursor copy_source_header = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-copy-source");
-    if (aws_http_headers_get(headers, copy_source_header, &source_header_value) != AWS_OP_SUCCESS) {
+    if (aws_http_headers_get(headers, copy_source_header, &source_bucket) != AWS_OP_SUCCESS) {
         AWS_LOGF_ERROR(AWS_LS_S3_GENERAL, "CopyRequest is missing the x-amz-copy-source header");
         return NULL;
     }
 
-    /* url decode */
-    struct aws_byte_buf decode_buffer;
-    AWS_ZERO_STRUCT(decode_buffer);
-    aws_byte_buf_init(&decode_buffer, allocator, 0);
-    if (aws_byte_buf_append_decoding_uri(&decode_buffer, &source_header_value) != AWS_OP_SUCCESS) {
-        goto error_cleanup;
-    }
-
-    /* extracts source bucket and key */
-    struct aws_byte_cursor source_bucket = aws_byte_cursor_from_buf(&decode_buffer);
-
-    /* x-amz-copy-source might have an optional leading slash. if so let's skip it */
-    if (decode_buffer.len > 1 && decode_buffer.buffer[0] == '/') {
+    if (source_bucket.len > 1 && source_bucket.ptr[0] == '/') {
         /* skip the leading slash */
         aws_byte_cursor_advance(&source_bucket, 1);
     }
-
     /* as we skipped the optional leading slash, from this point source format is always {bucket}/{key}. split them.
      */
     struct aws_byte_cursor source_key = source_bucket;
@@ -446,11 +481,9 @@ struct aws_http_message *aws_s3_get_source_object_size_message_new(
             "The CopyRequest x-amz-copy-source header must contain the bucket and object key separated by a slash");
         goto error_cleanup;
     }
-
     message = aws_s3_get_object_size_message_new(allocator, base_message, source_bucket, source_key);
 
 error_cleanup:
-    aws_byte_buf_clean_up(&decode_buffer);
     return message;
 }
 
@@ -491,11 +524,12 @@ struct aws_http_message *aws_s3_complete_multipart_message_new(
 
     const struct aws_byte_cursor *mpu_algorithm_checksum_name = aws_get_complete_mpu_name_from_algorithm(algorithm);
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
         allocator,
         base_message,
         g_s3_complete_multipart_upload_excluded_headers,
-        AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_excluded_headers));
+        AWS_ARRAY_SIZE(g_s3_complete_multipart_upload_excluded_headers),
+        true /*exclude_x_amz_meta*/);
 
     struct aws_http_headers *headers = NULL;
 
@@ -613,11 +647,12 @@ struct aws_http_message *aws_s3_abort_multipart_upload_message_new(
     struct aws_http_message *base_message,
     const struct aws_string *upload_id) {
 
-    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body(
+    struct aws_http_message *message = aws_s3_message_util_copy_http_message_no_body_filter_headers(
         allocator,
         base_message,
         g_s3_abort_multipart_upload_excluded_headers,
-        AWS_ARRAY_SIZE(g_s3_abort_multipart_upload_excluded_headers));
+        AWS_ARRAY_SIZE(g_s3_abort_multipart_upload_excluded_headers),
+        true /*exclude_x_amz_meta*/);
 
     if (aws_s3_message_util_set_multipart_request_path(allocator, upload_id, 0, false, message)) {
         goto error_clean_up;
@@ -760,11 +795,20 @@ error_clean_up:
 }
 
 /* Copy an existing HTTP message's headers, method, and path. */
-struct aws_http_message *aws_s3_message_util_copy_http_message_no_body(
+struct aws_http_message *aws_s3_message_util_copy_http_message_no_body_all_headers(
+    struct aws_allocator *allocator,
+    struct aws_http_message *base_message) {
+
+    return aws_s3_message_util_copy_http_message_no_body_filter_headers(allocator, base_message, NULL, 0, false);
+}
+
+struct aws_http_message *aws_s3_message_util_copy_http_message_no_body_filter_headers(
     struct aws_allocator *allocator,
     struct aws_http_message *base_message,
     const struct aws_byte_cursor *excluded_header_array,
-    size_t excluded_header_array_size) {
+    size_t excluded_header_array_size,
+    bool exclude_x_amz_meta) {
+
     AWS_PRECONDITION(allocator);
     AWS_PRECONDITION(base_message);
 
@@ -792,13 +836,37 @@ struct aws_http_message *aws_s3_message_util_copy_http_message_no_body(
         goto error_clean_up;
     }
 
-    size_t num_headers = aws_http_message_get_header_count(base_message);
+    if (aws_s3_message_util_copy_headers(
+            base_message, message, excluded_header_array, excluded_header_array_size, exclude_x_amz_meta)) {
+        goto error_clean_up;
+    }
+
+    return message;
+
+error_clean_up:
+
+    if (message != NULL) {
+        aws_http_message_release(message);
+        message = NULL;
+    }
+
+    return NULL;
+}
+
+int aws_s3_message_util_copy_headers(
+    struct aws_http_message *source_message,
+    struct aws_http_message *dest_message,
+    const struct aws_byte_cursor *excluded_header_array,
+    size_t excluded_header_array_size,
+    bool exclude_x_amz_meta) {
+
+    size_t num_headers = aws_http_message_get_header_count(source_message);
 
     for (size_t header_index = 0; header_index < num_headers; ++header_index) {
         struct aws_http_header header;
 
-        if (aws_http_message_get_header(base_message, &header, header_index)) {
-            goto error_clean_up;
+        if (aws_http_message_get_header(source_message, &header, header_index)) {
+            return AWS_OP_ERR;
         }
 
         if (excluded_header_array && excluded_header_array_size > 0) {
@@ -816,21 +884,18 @@ struct aws_http_message *aws_s3_message_util_copy_http_message_no_body(
             }
         }
 
-        if (aws_http_message_add_header(message, header)) {
-            goto error_clean_up;
+        if (exclude_x_amz_meta) {
+            if (aws_byte_cursor_starts_with_ignore_case(&header.name, &s_x_amz_meta_prefix)) {
+                continue;
+            }
+        }
+
+        if (aws_http_message_add_header(dest_message, header)) {
+            return AWS_OP_ERR;
         }
     }
 
-    return message;
-
-error_clean_up:
-
-    if (message != NULL) {
-        aws_http_message_release(message);
-        message = NULL;
-    }
-
-    return NULL;
+    return AWS_OP_SUCCESS;
 }
 
 /* Add a content-range header.*/
